/****************************************************************************
**
** Copyright (C) 2006-2009 fullmetalcoder <fullmetalcoder@hotmail.fr>
**
** This file is part of the Edyuk project <http://edyuk.org>
** 
** This file may be used under the terms of the GNU General Public License
** version 3 as published by the Free Software Foundation and appearing in the
** file GPL.txt included in the packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef _QDOCUMENT_MARK_H_
#define _QDOCUMENT_MARK_H_

/*!
	\file qdocumentmark.h
	\brief Definition of the QDocumentMark class.
	
	\see QDocumentMark
*/

#include "qce-config.h"

#include "qdocumentcursor.h"

#include <QDataStream>

class QDocumentCursorHandle;

class QDocumentMark
{
	friend class QMarksManager;
	friend class QDocumentCommand;
	
	public:
		enum Flag
		{
			ColumnMatters	= 1,
			ExpandOnType	= 2,
			Anchored		= 4
		};
		
		inline QDocumentMark()
		: m_flags(0), m_markType(-1), m_location(0)
		{}
		
		inline QDocumentMark(QDocumentCursorHandle* l, int t, char f = 0)
		: m_flags(f), m_markType(t), m_location(l)
		{}
		
		inline QDocumentMark(const QDocumentMark& m)
		: m_flags(0), m_markType(m.m_markType), m_location(m.m_location)
		{}
		
		inline char flags() const
		{ return m_flags; }
		
		inline void setFlags(char f)
		{ m_flags = f; }
		
		inline int type() const
		{ return m_markType; }
		
		inline void setType(int t)
		{ m_markType = t; }
		
		inline QDocumentCursor cursor() const
		{ return QDocumentCursor(m_location); }
		
		inline QDocumentCursorHandle* location() const
		{ return m_location; }
		
		inline int lineNumber() const
		{ return m_location ? m_location->lineNumber() : -1; }
		
		inline int columnNumber() const
		{ return m_location && (m_flags & ColumnMatters) ? m_location->columnNumber() : 0; }
		
		inline bool operator == (const QDocumentMark& m)
		{ return m_location ? m_location->eq(m.m_location) && (m_markType == m.m_markType) : false; }
		
		inline bool operator != (const QDocumentMark& m)
		{ return m_location ? !m_location->eq(m.m_location) || (m_markType != m.m_markType) : true; }
		
	private:
		char m_flags;
		int m_markType;
		QDocumentCursorHandle *m_location;
};

typedef QList<QDocumentMark*> QDocumentMarkList;

QCE_EXPORT QDataStream& operator >> (QDataStream& d, QDocumentMark *m);
QCE_EXPORT QDataStream& operator << (QDataStream& d, const QDocumentMark *m);

QCE_EXPORT QDataStream& operator >> (QDataStream& d, QDocumentMark& m);
QCE_EXPORT QDataStream& operator << (QDataStream& d, const QDocumentMark& m);

#endif
